import base64
import os


def display_data_for_image(filename, pid):
    with open(filename, 'rb') as f:
        image = f.read()
    os.unlink(filename)

    image_type = filename.split('.')[-1]
    if image_type == 'svg':
        image_type = 'image/svg+xml'
        image_data = image.decode('utf-8')
    elif image_type == 'csv':
        image_type = 'text/html'
        image = image.decode('utf-8')
        with open(filename + 'X', 'rb') as f:
            metadata = f.read().decode('utf-8').splitlines()
        os.unlink(filename + 'X')
        logscale = metadata[0].split(' ')[1]
        to_show = metadata[1].split(' ')[1:]
        against = metadata[2].split(' ')[1]
        lines = [line.split(',') for line in image.splitlines()]
        headers = lines.pop(0)[1:]
        i = 0
        labels = []
        while i < len(headers):
            if headers[i].startswith('"'):
                if headers[i].endswith('"'):
                    labels += headers[i]
                    i += 1
                else:
                    j = i + 1
                    while not headers[j].endswith('"'):
                        j += 1
                    labels.append(','.join(headers[i:j+1]))
                    i = j + 1
            else:
                labels.append('"{}"'.format(headers[i]))
                i += 1
        # remove quotations and store back in headers
        headers = [l[1:-1] for l in labels]
        labels = ', '.join(labels)
        if against in headers:
            xindex = headers.index(against) + 1
            # FIXME uses fugly hack in jquery.jqplot.js, patched at
            # shapeRenderer
            phase = 'true'
        else:
            against = 'Time'
            xindex = 0
            phase = 'false'
        series = '[' + ', '.join([
            '[' +
            ', '.join(['[' + lines[i][xindex] + ', ' + lines[i][j] + ']'
                       for i in range(len(lines))]) +
            ']' for j in range(1, len(lines[0]))]) + ']'
        plotid = filename.split('.')[-2] + '-{pid}'.format(pid=pid)
        axes = [
            (
                axis,
                'label: "{xlabel}",\n'.format(xlabel=against) + ' '*32
                if axis == 'x'
                else '',
                'renderer: $.jqplot.{log}AxisRenderer'.format(
                    log='Log' if axis in logscale else 'Linear')
            ) for axis in 'xy'
        ]
        axes = ','.join([
            '\n' + ' '*28 +
            '''{axis}axis: {{
                                {label}{renderer},
                                tickOptions: {{
                                    formatString: "%.3p"
                                }}
                            }}'''.format(axis=axis,
                                         label=label,
                                         renderer=renderer)
            for (axis, label, renderer) in axes
        ])
        if to_show == ['{}'] or not [h for h in headers if h in to_show]:
            plotted_series = ''
        else:
            plotted_series = '\n' + ' '*25 + 'series: [' + ', '.join(
                ['{show: ' + str(label in to_show).lower() + '}'
                 for label in headers]
            ) + '],'
        image_data = '''
        <div id="{plotid}" style="height: 400px; width: 600px;"></div>
        <script>
            function waitForJQplot() {{
                if (typeof $.jqplot !== 'undefined') {{
                    $.jqplot.config.enablePlugins = true;
                    $.jqplot.config.phase = {phase};
                    $.jqplot('{plotid}', {data}, {{
                        seriesDefaults: {{
                            showMarker: false
                        }},
                        axes: {{{axes}
                        }},{plotted_series}
                        legend: {{
                            show: true,
                            placement: "outside",
                            labels: [{labels}],
                            renderer: $.jqplot.EnhancedLegendRenderer,
                            rendererOptions: {{
                                seriesToggle: true,
                                showLineStyle: true
                            }}
                        }},
                        sortData: false,
                        cursor: {{
                            show: true,
                            zoom: true,
                            tooltipFormatString: "%.3p, %.3p",
                            cursorLegendFormatString: "%.3p, %.3p"
                        }},
                        highlighter: {{
                            formatString: "%.3p"
                        }}
                    }});
                    $('#{plotid}').contextmenu(function() {{
                        $('#{plotid}').jqplotSaveImage();
                        return false;
                    }});
                    // dirty way to insert image as secondary data in current
                    // output area
                    window.setTimeout(
                        function () {{
                            $('#{plotid}').closest('.cell').data('cell').
                                output_area.outputs[1].data['image/png'] =
                            $('#{plotid}').jqplotToImageStr({{}}).substring(22);
                        }}, 1000
                    );
                }} else {{ window.setTimeout(waitForJQplot, 250); }}
            }}
            $(document).ready(function(){{
                waitForJQplot();
            }});
        </script>
        '''.format(plotid=plotid, data=series, axes=axes,
                   plotted_series=plotted_series,
                   labels=labels, phase=phase)
    else:
        image_type = 'image/' + image_type
        image_data = base64.b64encode(image).decode('ascii')

    content = {
        'data': {
            image_type: image_data
        },
        'metadata': {}
    }
    return content


def extract_image_filenames(output):
    output_lines = []
    image_filenames = []

    for line in output.split("\n"):
        if line.startswith('<img src="'):
            filename = line.rstrip().split('"')[1]
            image_filenames.append(filename)
        else:
            output_lines.append(line)

    output = "\n".join(output_lines)
    return image_filenames, output
